import ModelComponent from "../../db/ModelComponent";
import User from "../models/User";
import config from "config";

export const USERNAME_REGEXP = /^[0-9a-z_-]+$/;

export default class UserNameComponent extends ModelComponent<User> {
    private name?: string = undefined;
    private name_changed_at?: Date | null = undefined;

    public init(): void {
        this.setValidation('name').defined().between(3, 64).regexp(USERNAME_REGEXP).unique(this._model);
    }

    public hasName(): boolean {
        return !!this.name;
    }

    public getName(): string {
        if (!this.name) throw new Error('User has no name.');
        return this.name;
    }

    public setName(newName: string): boolean {
        if (!this.canChangeName()) return false;

        this.name = newName;
        this.name_changed_at = new Date();
        return true;
    }

    public getNameChangedAt(): Date | null {
        return this.name_changed_at || null;
    }

    public forgetNameChangeDate(): void {
        this.name_changed_at = null;
    }

    public canChangeName(): boolean {
        return !this.name_changed_at ||
            Date.now() - this.name_changed_at.getTime() >= config.get<number>('auth.name_change_wait_period');
    }
}
